/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | HZDR Multiphase Addon for OpenFOAM
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2021 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "WeberNumberModel.H"
#include "phasePair.H"

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::sigmaEff() const
{
    return pair_.sigma() + sigmat();
}


Foam::tmp<Foam::volVectorField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::subGridSurfaceTensionForce
() const
{
    return sigmat()*pair_.K()*pair_.gradAlpha();
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::sigmat() const
{
    tmp<volScalarField> sigmaCrit
    (
        pair_.rho()*magSqr(U())*sqr(delta())*mag(pair_.K())/WeCrit()
    );

    return
        max
        (
            sigmaCrit - pair_.sigma(),
            dimensionedScalar(pair_.sigma()().dimensions(), Zero)
        );
}


Foam::tmp<Foam::surfaceScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::sigmatf() const
{
    tmp<surfaceScalarField> sigmaCritf
    (
        fvc::interpolate(pair_.rho())*fvc::interpolate(magSqr(U()))
       *magSqr(mesh_.delta())*fvc::interpolate(mag(pair_.K()))
       /fvc::interpolate(WeCrit())
    );

    return
        max
        (
            sigmaCritf - fvc::interpolate(pair_.sigma()),
            dimensionedScalar(pair_.sigma()().dimensions(), Zero)
        );
}


Foam::tmp<Foam::volVectorField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::U() const
{
    const phaseModel& phase1 = pair_.phase1();
    const phaseModel& phase2 = pair_.phase2();

    return
        phase1*phase1.U() + phase2*phase2.U()
       /max
        (
            phase1 + phase2,
            phase1.residualAlpha() + phase2.residualAlpha()
        );
}


bool Foam::subGridSurfaceTensionModels::WeberNumberModel::write() const
{
    if (subGridSurfaceTensionModel::write())
    {
        tmp<volScalarField> sigmaEffective
        (
            volScalarField::New
            (
                IOobject::groupName("sigmaEff", pair_.name()),
                sigmaEff()
            )
        );
        sigmaEffective.ref().write();

        return true;
    }
    else
    {
        return false;
    }
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModels::WeberNumberModel::WeberNumberModel
(
    const dictionary& dict,
    const phasePair& pair
)
:
    subGridSurfaceTensionModel(dict, pair),

    delta_
    (
        SGSdelta::New
        (
            IOobject::groupName("delta", pair.name()),
            mesh_,
            dict
        )
    )
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::subGridSurfaceTensionModels::WeberNumberModel::~WeberNumberModel()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::surfaceScalarField>
Foam::subGridSurfaceTensionModels::WeberNumberModel::subGridSurfaceTensionForcef
() const
{
    if (debug && mesh_.time().outputTime()) write();

    return sigmatf()*fvc::interpolate(pair_.K())*pair_.gradAlphaf();
}


// ************************************************************************* //
