/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | HZDR Multiphase Addon for OpenFOAM
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | Copyright (C) 2022 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "accumulationToContinuousZhang.H"
#include "phasePair.H"
#include "phaseSystem.H"
#include "addToRunTimeSelectionTable.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace phaseTransferModels
{
    defineTypeNameAndDebug(accumulationToContinuousZhang, 0);
    addToRunTimeSelectionTable(
        phaseTransferModel, accumulationToContinuousZhang, dictionary
    );
}
}


// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::accumulationToContinuousZhang::rate(
    const phaseModel& pairDisperse,
    const phaseModel& pairContinuous
) const
{
    tmp<volScalarField> tRate
    (
        volScalarField::New
        (
            IOobject::groupName("dmdtf."+typeName, pair_.name()),
            pairContinuous.mesh(),
            dimensionedScalar(dimDensity/dimTime, Zero)
        )
    );

    tRate.ref() =
        pairDisperse*pairDisperse.rho()
        /(nDeltaT_*pairDisperse.fluid().time().deltaT());

    return tRate;
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::phaseTransferModels::accumulationToContinuousZhang
    ::accumulationToContinuousZhang
(
    const dictionary& dict,
    const phasePair& pair
)
:
    phaseTransferModel(dict, pair),
    pair_(pair),
    continuousPhases_(pair_.phase1().fluid().continuousPhases()),
    nDeltaT_(dict.lookupOrDefault<scalar>("nDeltaT", 50)),
    alphaCrit_
    (
        dimensionedScalar::lookupOrDefault
        (
            "alphaCrit",
            dict,
            dimless,
            0.5
        )
    ),
    alphaDelta_
    (
      dimensionedScalar::lookupOrDefault
      (
         "alphaDelta",
          dict,
          dimless,
          0.05
      )
    ),
    pairContinuousThreshold_
    (
        dimensionedScalar::lookupOrDefault
        (
            "pairContinuousThreshold",
            dict,
            dimless,
            0.05
        )
    )
{
    if ((!pair_.phase1().continuous()) && (!pair_.phase2().continuous()))
    {
        FatalErrorInFunction
            << "A continuous phase is expected in the pair "
            << pair_.name()
            << " for transition to continuous by threshold value"
            << exit(FatalError);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::phaseTransferModels::accumulationToContinuousZhang
    ::~accumulationToContinuousZhang()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

bool Foam::phaseTransferModels::accumulationToContinuousZhang
    ::mixture() const
{
    return true;
}


Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::accumulationToContinuousZhang::dmdtf() const
{
    tmp<volScalarField> tDmdtf
    (
        volScalarField::New
        (
            IOobject::groupName("dmdtf."+typeName, pair_.name()),
            pair_.phase1().mesh(),
            dimensionedScalar(dimDensity/dimTime, Zero)
        )
    );

    const phaseModel* pairDispersePtr = nullptr;
    const phaseModel* pairContinuousPtr = nullptr;

    if (pair_.phase1().continuous())
    {
        pairDispersePtr = &pair_.phase2();
        pairContinuousPtr = &pair_.phase1();
    }
    else
    {
        pairDispersePtr = &pair_.phase1();
        pairContinuousPtr = &pair_.phase2();
    }

    const phaseModel& pairDisperse = *pairDispersePtr;
    const phaseModel& pairContinuous = *pairContinuousPtr;
    const scalar sign = pair_.phase1().continuous() ? 1 : -1;

    const volScalarField A
    (
        min
        (
            max
            (
                (pairDisperse - alphaCrit_ + alphaDelta_)
                /(2*alphaDelta_),
                0.0
            ),
            1.0
        )
    );

    tDmdtf.ref() =
        sign*rate(pairDisperse, pairContinuous)
       *max
        (
            3*sqr(A) - 2*pow3(A),
            pos0(pairContinuous - pairContinuousThreshold_)
        );

    if (debug && pair_.phase1().time().outputTime())
    {
        tDmdtf.ref().write();
    }

    return tDmdtf;
}


// ************************************************************************* //
