/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | HZDR Multiphase Addon for OpenFOAM
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | Copyright (C) 2022 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::phaseTransferModels::accumulationToContinuousZhang

Description
    Phase transfer model representing change from a dispersed phase to a
    continuous phase with identical physical quantities. This change is
    triggered by a threshold value for the void fraction sum of the gas phase
    pair.

    The keyword 'alphaThreshold' is the threshold value for the void
    fraction sum of the gas phase. If exceeded, the transfer from dispersed to
    continuous starts.

Usage
    \table
        Property       | Description                       | Required | Default
        nDeltaT        | Time steps for complete transfer  | no       | 50
        alphaCrit      | Disperse threshold                | no       | 0.5
        alphaDelta     | Transition region parameter       | no       | 0.05
        pairContinuousThreshold | Minimum continuous fraction | no    | 0.05
    \endtable

SourceFiles
    accumulationToContinuousZhang.C

\*---------------------------------------------------------------------------*/

#ifndef accumulationToContinuousZhang_H
#define accumulationToContinuousZhang_H

#include "phaseTransferModel.H"
#include "phaseModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class phasePair;

namespace phaseTransferModels
{

/*---------------------------------------------------------------------------*\
                         Class accumulationToContinuousZhang Declaration
\*---------------------------------------------------------------------------*/

class accumulationToContinuousZhang
:
    public phaseTransferModel
{
private:

    // Private typedef

        typedef UPtrList<phaseModel> phaseModelPartialList;


    // Private Data

        //- Phase pair
        const phasePair& pair_;

        //- Name list of the continuous phases
        const phaseModelPartialList continuousPhases_;

        //- Number of time steps to completely transfer volume fraction
        const scalar nDeltaT_;

        //- Threshold for volume fraction
        const dimensionedScalar alphaCrit_;
        const dimensionedScalar alphaDelta_;
        const dimensionedScalar pairContinuousThreshold_;

    // Private member functions

        //- Return the accumulationToContinuousZhang rate
        Foam::tmp<volScalarField> rate(
            const phaseModel& pairDisperse,
            const phaseModel& pairContinuous
        ) const;


public:

    //- Runtime type information
    TypeName("accumulationToContinuousZhang");


    // Constructors

        //- Construct from components
        accumulationToContinuousZhang
        (
            const dictionary& dict,
            const phasePair& pair
        );


    //- Destructor
    virtual ~accumulationToContinuousZhang();


    // Member Functions

        //- Whether or not there is a mass transfer rate for the mixture
        virtual bool mixture() const;

        //- The mass transfer rate for the mixture
        virtual tmp<volScalarField> dmdtf() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace phaseTransferModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
