/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | HZDR Multiphase Addon for OpenFOAM
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | Copyright (C) 2019-2022 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "detrainment.H"
#include "addToRunTimeSelectionTable.H"
#include "phaseSystem.H"

// * * * * * * * * * * * * * * Static Data Members * * * * * * * * * * * * * //

namespace Foam
{
namespace phaseTransferModels
{
    defineTypeNameAndDebug(detrainment, 0);
    addToRunTimeSelectionTable(phaseTransferModel, detrainment, dictionary);
}
}


// * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::detrainment::rate
(
    const phaseModel& phase
) const
{
    return phase.rho()/(nDeltaT_*phase.fluid().time().deltaT());
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::phaseTransferModels::detrainment::detrainment
(
    const dictionary& dict,
    const phasePair& pair
)
:
    morphologyTransferModel(dict, pair),
    pairDisperse_(pair.phase1()),
    pairContinuous_(pair.phase2()),
    nDeltaT_(dict.lookupOrDefault<scalar>("nDeltaT", 100)),
    pairContinuousRatioThreshold_
    (
        dimensionedScalar::lookupOrDefault
        (
            "pairContinuousRatioThreshold",
            dict,
            dimless,
            0.9
        )
    )
{
    if (pair.phase1().continuous() || !pair.phase2().continuous())
    {
        FatalErrorInFunction
            << "Expecting an ordered pair of (disperse in continuous) "
            << "in pair " << pair.name() << " for " << typeName << " model."
            << exit(FatalError);
    }
}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

Foam::phaseTransferModels::detrainment::~detrainment()
{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

Foam::tmp<Foam::volScalarField>
Foam::phaseTransferModels::detrainment::dmdtf() const
{
    const volScalarField pairContinuousRatio
    (
        pairContinuous_/max(pairContinuous_.fluid().sumAlphaContinuous(), small)
    );

    const fvMesh& mesh(pair_.phase1().mesh());

    tmp<volScalarField> tDmdtf
    (
        new volScalarField
        (
            IOobject
            (
                IOobject::groupName("dmdtf."+typeName, pair_.name()),
                mesh.time().timeName(),
                mesh
            ),
          - pos0(pairContinuousRatio - pairContinuousRatioThreshold_)
           *pos(pairDisperse_ - pairDisperse_.residualAlpha())
           *rate(pairDisperse_)
        )
    );

    if (debug && mesh.time().outputTime()) tDmdtf.ref().write();

    return tDmdtf;
}


// ************************************************************************* //
