/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | HZDR Multiphase Addon for OpenFOAM
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | Copyright (C) 2022 OpenFOAM Foundation
  ==  == ====== ====   ==  ==  | and Helmholtz-Zentrum Dresden-Rossendorf
-------------------------------------------------------------------------------
License
    This file is a derivative work of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::wallLubricationModels::Shi

Description
    Wall lubrication model of Shi et al. (2021).

    References:
    The original model introduced by (Eq. 6a, b and c):
    \verbatim
        Takemura, F. & Magnaudet, J. (2003)
        The transverse force on clean and contaminated bubbles rising near a
        vertical wall at moderate Reynolds number.
        Journal of Fluid Mechanics, 495, 235-253.
    \endverbatim

    with modifications from (Eq. 20, 24, 25b and 26):
    \verbatim
        Shi, P., Rzehak, R., Lucas, D., & Magnaudet, J. (2021)
        Drag and lift forces on a rigid sphere immersed in a wall-bounded linear
        shear flow.
        Physical Review Fluids, 6, 104309.
    \endverbatim

    Note that the slip-induced transverse force in a fluid at rest at
    moderate-to-large Re with assumption for Low-Re wall-bounded shear flow is
    implemented. Also the model is based on data of 0.1 <= Re <= 100. Therefore,
    Re is bounded in this range. Furthermore, a conversion is required from
    Takemura & Magnaudet (2003) definition of \f$C_{W}^{*}\f$ to OpenFOAM's
    definition of \f$C_{W}\f$:

    \f[
        \frac{\vec{F}_{W}}{\vec{F}_{W}^{*}}
        = \frac
          {
              C_{W} \frac{\pi}{3} d_P^2 \rho_L u_{rel}^2 \vec{n}
          }
          {
              C_{W}^{*} \frac{\pi}{8} d_P^2 \rho_L u_{rel}^2 \vec{n}
          }
        = 1
    \f]

SourceFiles
    Shi.C

\*---------------------------------------------------------------------------*/

#ifndef Shi_H
#define Shi_H

#include "wallLubricationModel.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

class phasePair;

namespace wallLubricationModels
{

/*---------------------------------------------------------------------------*\
                             Class Shi Declaration
\*---------------------------------------------------------------------------*/

class Shi
:
    public wallLubricationModel
{
    // Private Data

        //- Residual Reynolds Number
        const dimensionedScalar residualRe_;


public:

    //- Runtime type information
    TypeName("Shi");


    // Constructors

        //- Construct from components
        Shi
        (
            const dictionary& dict,
            const phasePair& pair
        );


    //- Destructor
    virtual ~Shi();


    // Member Functions

        //- Return phase-intensive wall lubrication force
        tmp<volVectorField> Fi() const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace wallLubricationModels
} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

#endif

// ************************************************************************* //
